/*
 * The Clear BSD License
 * Copyright (c) 2013 - 2015, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted (subject to the limitations in the disclaimer below) provided
 *  that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*******************************************************************************
* File Name          : lcd.c
* Version            : V1.1.0
* Date               : 11/13/2009
* Description        : Manley zymG12864C3
*******************************************************************************/
#include "board.h"
#include "fsl_gpio.h"
#include "fsl_port.h"
#include "lcd.h"
#include "fsl_dspi.h"

#define EXAMPLE_DSPI_MASTER_PCS_NUMBER (2U)

const unsigned char FONTTYPE6_8[][8] = {
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // - -
    {0x20, 0x20, 0x20, 0x20, 0x20, 0x00, 0x20, 0x00}, // -!-
    {0x50, 0x50, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00}, // -"-
    {0x50, 0x50, 0xF8, 0x50, 0xF8, 0x50, 0x50, 0x00}, // -#-
    {0x20, 0x78, 0xC0, 0x70, 0x28, 0xF0, 0x20, 0x00}, // -$-
    {0xC0, 0xC8, 0x10, 0x20, 0x40, 0x98, 0x18, 0x00}, // -%-
    {0x40, 0xA0, 0xA0, 0x40, 0xA8, 0x90, 0x68, 0x00}, // -&-
    {0x30, 0x20, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00}, // -'-
    {0x10, 0x20, 0x40, 0x40, 0x40, 0x20, 0x10, 0x00}, // -(-
    {0x40, 0x20, 0x10, 0x10, 0x10, 0x20, 0x40, 0x00}, // -)-
    {0x20, 0xA8, 0x70, 0x20, 0x70, 0xA8, 0x20, 0x00}, // -*-
    {0x20, 0x20, 0x20, 0xF8, 0x20, 0x20, 0x20, 0x00}, // -+-
    {0x00, 0x00, 0x00, 0x00, 0x60, 0x40, 0x80, 0x00}, // -,-
    {0x00, 0x00, 0x00, 0xF8, 0x00, 0x00, 0x00, 0x00}, // ---
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x60, 0x00}, // -.-
    {0x00, 0x08, 0x10, 0x20, 0x40, 0x80, 0x00, 0x00}, // -/-
    {0x70, 0x88, 0x98, 0xA8, 0xC8, 0x88, 0x70, 0x00}, // -0-
    {0x20, 0x60, 0x20, 0x20, 0x20, 0x20, 0x70, 0x00}, // -1-
    {0x70, 0x88, 0x08, 0x30, 0x40, 0x80, 0xF8, 0x00}, // -2-
    {0xF8, 0x08, 0x10, 0x30, 0x08, 0x88, 0x70, 0x00}, // -3-
    {0x10, 0x30, 0x50, 0x90, 0xF8, 0x10, 0x10, 0x00}, // -4-
    {0xF8, 0x80, 0xF0, 0x08, 0x08, 0x88, 0x70, 0x00}, // -5-
    {0x38, 0x40, 0x80, 0xF0, 0x88, 0x88, 0x70, 0x00}, // -6-
    {0xF8, 0x08, 0x10, 0x20, 0x40, 0x40, 0x40, 0x00}, // -7-
    {0x70, 0x88, 0x88, 0x70, 0x88, 0x88, 0x70, 0x00}, // -8-
    {0x70, 0x88, 0x88, 0x78, 0x08, 0x10, 0xE0, 0x00}, // -9-
    {0x00, 0x60, 0x60, 0x00, 0x60, 0x60, 0x00, 0x00}, // -:-
    {0x00, 0x60, 0x60, 0x00, 0x60, 0x60, 0x80, 0x00}, // -;-
    {0x10, 0x20, 0x40, 0x80, 0x40, 0x20, 0x10, 0x00}, // -<-
    {0x00, 0x00, 0xF8, 0x00, 0xF8, 0x00, 0x00, 0x00}, // -=-
    {0x40, 0x20, 0x10, 0x08, 0x10, 0x20, 0x40, 0x00}, // ->-
    {0x70, 0x88, 0x10, 0x20, 0x20, 0x00, 0x20, 0x00}, // -?-
    {0x70, 0x88, 0xB8, 0xA8, 0xB8, 0x80, 0x78, 0x00}, // -@-
    {0x20, 0x50, 0x88, 0x88, 0xF8, 0x88, 0x88, 0x00}, // -A-
    {0xF0, 0x88, 0x88, 0xF0, 0x88, 0x88, 0xF0, 0x00}, // -B-
    {0x70, 0x88, 0x80, 0x80, 0x80, 0x88, 0x70, 0x00}, // -C-
    {0xF0, 0x88, 0x88, 0x88, 0x88, 0x88, 0xF0, 0x00}, // -D-
    {0xF8, 0x80, 0x80, 0xF0, 0x80, 0x80, 0xF8, 0x00}, // -E-
    {0xF8, 0x80, 0x80, 0xF0, 0x80, 0x80, 0x80, 0x00}, // -F-
    {0x70, 0x88, 0x80, 0x80, 0xB8, 0x88, 0x78, 0x00}, // -G-
    {0x88, 0x88, 0x88, 0xF8, 0x88, 0x88, 0x88, 0x00}, // -H-
    {0x70, 0x20, 0x20, 0x20, 0x20, 0x20, 0x70, 0x00}, // -I-
    {0x38, 0x10, 0x10, 0x10, 0x10, 0x90, 0x60, 0x00}, // -J-
    {0x88, 0x90, 0xA0, 0xC0, 0xA0, 0x90, 0x88, 0x00}, // -K-
    {0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0xF8, 0x00}, // -L-
    {0x88, 0xD8, 0xA8, 0xA8, 0x88, 0x88, 0x88, 0x00}, // -M-
    {0x88, 0x88, 0xC8, 0xA8, 0x98, 0x88, 0x88, 0x00}, // -N-
    {0x70, 0x88, 0x88, 0x88, 0x88, 0x88, 0x70, 0x00}, // -O-
    {0xF0, 0x88, 0x88, 0xF0, 0x80, 0x80, 0x80, 0x00}, // -P-
    {0x70, 0x88, 0x88, 0x88, 0xA8, 0x90, 0x68, 0x00}, // -Q-
    {0xF0, 0x88, 0x88, 0xF0, 0xA0, 0x90, 0x88, 0x00}, // -R-
    {0x70, 0x88, 0x80, 0x70, 0x08, 0x88, 0x70, 0x00}, // -S-
    {0xF8, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x00}, // -T-
    {0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x70, 0x00}, // -U-
    {0x88, 0x88, 0x88, 0x88, 0x88, 0x50, 0x20, 0x00}, // -V-
    {0x88, 0x88, 0x88, 0xA8, 0xA8, 0xD8, 0x88, 0x00}, // -W-
    {0x88, 0x88, 0x50, 0x20, 0x50, 0x88, 0x88, 0x00}, // -X-
    {0x88, 0x88, 0x50, 0x20, 0x20, 0x20, 0x20, 0x00}, // -Y-
    {0xF8, 0x08, 0x10, 0x20, 0x40, 0x80, 0xF8, 0x00}, // -Z-
    {0xF0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xF0, 0x00}, // -[-
    {0x00, 0x80, 0x40, 0x20, 0x10, 0x08, 0x00, 0x00}, // -\-
    {0x78, 0x18, 0x18, 0x18, 0x18, 0x18, 0x78, 0x00}, // -]-
    {0x20, 0x70, 0xA8, 0x20, 0x20, 0x20, 0x20, 0x00}, // -^-
    {0x00, 0x20, 0x40, 0xF8, 0x40, 0x20, 0x00, 0x00}, // -_-
    {0x20, 0x10, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00}, // -`-
    {0x00, 0x00, 0xE0, 0x10, 0x70, 0x90, 0x68, 0x00}, // -a-
    {0x80, 0x80, 0xB0, 0xC8, 0x88, 0xC8, 0xB0, 0x00}, // -b-
    {0x00, 0x00, 0x70, 0x88, 0x80, 0x80, 0x70, 0x00}, // -c-
    {0x08, 0x08, 0x68, 0x98, 0x88, 0x98, 0x68, 0x00}, // -d-
    {0x00, 0x00, 0x70, 0x88, 0xF0, 0x80, 0x70, 0x00}, // -e-
    {0x30, 0x48, 0x40, 0xF0, 0x40, 0x40, 0x40, 0x00}, // -f-
    {0x00, 0x00, 0x70, 0x88, 0x88, 0x78, 0x08, 0xF0}, // -g-
    {0x80, 0x80, 0xB0, 0xC8, 0x88, 0x88, 0x88, 0x00}, // -h-
    {0x20, 0x00, 0x00, 0x20, 0x20, 0x20, 0x20, 0x00}, // -i-
    {0x10, 0x00, 0x00, 0x30, 0x10, 0x10, 0x10, 0x60}, // -j-
    {0x80, 0x80, 0x90, 0xA0, 0xC0, 0xA0, 0x98, 0x00}, // -k-
    {0x60, 0x20, 0x20, 0x20, 0x20, 0x20, 0x70, 0x00}, // -l-
    {0x00, 0x00, 0x50, 0xA8, 0xA8, 0xA8, 0xA8, 0x00}, // -m-
    {0x00, 0x00, 0xB0, 0x48, 0x48, 0x48, 0x48, 0x00}, // -n-
    {0x00, 0x00, 0x70, 0x88, 0x88, 0x88, 0x70, 0x00}, // -o-
    {0x00, 0x00, 0xF0, 0x88, 0x88, 0xF0, 0x80, 0x80}, // -p-
    {0x00, 0x00, 0x78, 0x88, 0x88, 0x78, 0x08, 0x08}, // -q-
    {0x00, 0x00, 0xB0, 0x48, 0x40, 0x40, 0x40, 0x00}, // -r-
    {0x00, 0x00, 0x78, 0x80, 0x70, 0x08, 0xF0, 0x00}, // -s-
    {0x40, 0x40, 0xF8, 0x40, 0x40, 0x48, 0x30, 0x00}, // -t-
    {0x00, 0x00, 0x90, 0x90, 0x90, 0x90, 0x68, 0x00}, // -u-
    {0x00, 0x00, 0x88, 0x88, 0x88, 0x50, 0x20, 0x00}, // -v-
    {0x00, 0x00, 0xA8, 0xA8, 0xA8, 0xA8, 0x50, 0x00}, // -w-
    {0x00, 0x00, 0x88, 0x50, 0x20, 0x50, 0x88, 0x00}, // -x-
    {0x00, 0x00, 0x88, 0x88, 0x98, 0x68, 0x08, 0xF0}, // -y-
    {0x00, 0x00, 0xF8, 0x10, 0x20, 0x40, 0xF8, 0x00}, // -z-
    {0x20, 0x40, 0x40, 0x80, 0x40, 0x40, 0x20, 0x00}, // -{-
    {0x20, 0x20, 0x20, 0x00, 0x20, 0x20, 0x20, 0x00}, // -|-
    {0x20, 0x10, 0x10, 0x08, 0x10, 0x10, 0x20, 0x00}, // -}-
    {0x00, 0x00, 0x40, 0xA8, 0x10, 0x00, 0x00, 0x00}, // -~-
    {0xA8, 0x50, 0xA8, 0x50, 0xA8, 0x50, 0xA8, 0x00}, // --
};

static unsigned char sUpdate_mode = 1;

static unsigned char gui_disp_buf[GUI_LCM_YMAX / 8][GUI_LCM_XMAX];

const unsigned char BIT_MASK[8] = {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};

/* simple font: ' ', '0'~'9','a'~'z','A'~'Z' */
extern const unsigned char FONTTYPE8_8[][8];

/*******************************************************************************
* Function Name  : LCD_Spi_Send
* Description    : spi send data
* Input          : None.
* Return         : None.
*******************************************************************************/
static void LCD_Spi_Send(unsigned char value)
{
    SPI0->PUSHR = value | ((1U << EXAMPLE_DSPI_MASTER_PCS_NUMBER) << 16);
    while ((SPI0->SR & SPI_SR_TCF_MASK) != SPI_SR_TCF_MASK)
        ;
    SPI0->SR |= SPI_SR_TCF_MASK;
}

/*******************************************************************************
* Function Name  : LCD_Set_Update_Mode
* Description    : set mode
* Input          : None.
* Return         : None.
*******************************************************************************/
void LCD_Set_Update_Mode(unsigned char mode)
{
    sUpdate_mode = mode;
}

/*******************************************************************************
* Function Name  : Delay
* Description    : Delay some time.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
static void Delay(void)
{
    int i = 0xf;
    while (i--)
        __asm("nop");
}

/*******************************************************************************
* Function Name  : Reset_Delay
* Description    : Delay some time
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
static void Reset_Delay(void)
{
    int i = 0xff;
    while (i--)
        __asm("nop");
}

/*******************************************************************************
* Function Name  : Power_Delay
* Description    : Delay some time for power up.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
static void Power_Delay(void)
{
    unsigned long i = 0x4fff;
    while (i--)
        __asm("nop");
}

/*******************************************************************************
* Function Name  : LCD_Init
* Description    : initialize lcd
* Input          : None.
* Return         : None.
*******************************************************************************/
void LCD_Initialize(void)
{
    Power_Delay();
    LCD_WriteCmd(Display_Off);
    Reset_Delay();
    // Resetting circuit
    LCD_WriteCmd(Reset_LCD);
    Reset_Delay();
    // LCD bias setting
    LCD_WriteCmd(Set_LCD_Bias_9);
    Reset_Delay();
    // ADC selection: display from left to right
    LCD_WriteCmd(Set_ADC_Normal);
    Reset_Delay();
    // Common output state selection: display from up to down
    LCD_WriteCmd(COM_Scan_Dir_Reverse);
    Reset_Delay();

    LCD_WriteCmd(Power_Booster_On);
    Power_Delay(); // 50ms requried
    LCD_WriteCmd(Power_Regulator_On);
    Power_Delay(); // 50ms
    LCD_WriteCmd(Power_Follower_On);
    Power_Delay(); // 50ms

    // Setting the built-in resistance radio for regulation of the V0 voltage
    // Electronic volume control
    // Power control setting
    LCD_WriteCmd(Set_ElecVol_Reg | 0x05);
    Delay();
    LCD_WriteCmd(Set_ElecVol_Mode);
    Delay();
    LCD_WriteCmd(Set_ElecVol_Reg);
    Delay();
    LCD_WriteCmd(Set_Page_Addr_0);
    Delay();
    LCD_WriteCmd(Set_ColH_Addr_0);
    Delay();
    LCD_WriteCmd(Set_ColL_Addr_0);
    Delay();
    LCD_WriteCmd(Display_On);
    Delay();

    LCD_WriteCmd(Display_All_On);
    Delay();
    LCD_WriteCmd(Display_Off);
    Delay();
    LCD_WriteCmd(Display_On);
    Delay();
    LCD_WriteCmd(Display_All_Normal);
    Delay();
    LCD_ClearSCR();
}

/*******************************************************************************
* Function Name  : LCD_PowerOn
* Description    : Power on the LCD.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_Power_On(void)
{
    LCD_WriteCmd(Display_On);
    Delay();
    LCD_WriteCmd(Power_Booster_On);
    Delay();
    LCD_WriteCmd(Power_Regulator_On);
    Delay();
    LCD_WriteCmd(Power_Follower_On);
    Delay();
}

/*******************************************************************************
* Function Name  : LCD_PowerOff
* Description    : Power off the LCD.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_Power_Off(void)
{
    LCD_WriteCmd(Display_Off); // Power saver
    Delay();
    LCD_WriteCmd(Display_All_On); // commands(compound)
    Delay();
}

/*******************************************************************************
* Function Name  : LCD_Sleep_Enter
* Description    : Enter the sleep mode.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_Sleep_Enter(void)
{
    // Sleep Mode Setting (Sleep Mode)
    LCD_WriteCmd(Sleep_Mode_Enable); // preceding Command
    Delay();
    LCD_WriteCmd(Sleep_Mode_Deliver); // pollowing Command
    Delay();
    LCD_WriteCmd(Display_Off); // power saver
    Delay();
}

/*******************************************************************************
* Function Name  : LCD_Sleep_Exit
* Description    : Exit the sleep mode.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_Sleep_Exit(void)
{
    LCD_WriteCmd(Display_All_Normal); // display all points OFF
    Delay();
    // Sleep Mode Setting (Normal Mode)
    LCD_WriteCmd(Sleep_Mode_Disable); // preceding Command
    Delay();
    LCD_WriteCmd(Sleep_Mode_Deliver); // pollowing Command
    Delay();
    LCD_WriteCmd(Display_On);
    Delay();
}

/*******************************************************************************
* Function Name  : LCD_DisplayOn
* Description    : Enables the Display.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_DisplayOn(void)
{
    LCD_WriteCmd(Display_On);
}

/*******************************************************************************
* Function Name  : LCD_DisplayOff
* Description    : Disables the Display.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_DisplayOff(void)
{
    LCD_WriteCmd(Display_Off);
}

/*******************************************************************************
* Function Name  : LCD_FillAll
* Description    : Fill the whole LCD.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_FillAll(unsigned char *data)
{
    unsigned char i, j = GUI_LCM_XMAX;
    unsigned char *p = data;

    for (i = 0; i < GUI_LCM_PAGE; i++)
    {
        LCD_WriteCmd(Set_Page_Addr_0 | i);
        LCD_WriteCmd(Set_ColH_Addr_0);
        LCD_WriteCmd(Set_ColL_Addr_0);
        j = GUI_LCM_XMAX;
        while (j--)
        {
            LCD_WriteData(*p++);
            Delay();
            Delay();
        }
    }
}

/*******************************************************************************
* Function Name  : LCD_FillUpdate
* Description    : Fill the whole LCD.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_FillUpdate(unsigned char *data)
{
    unsigned char i, j = GUI_LCM_XMAX;
    unsigned char *p = &gui_disp_buf[0][0];

    for (i = 0; i < GUI_LCM_PAGE; i++)
    {
        j = GUI_LCM_XMAX;
        while (j--)
        {
            *p++ = *data++;
        }
    }
    LCD_FillAll((unsigned char *)gui_disp_buf);
}

/*******************************************************************************
* Function Name  : LCD_ClearSCR
* Description    : clean screen
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_ClearSCR(void)
{
    unsigned char i;
    unsigned char j;

    for (i = 0; i < GUI_LCM_PAGE; i++)
    {
        for (j = 0; j < GUI_LCM_XMAX; j++)
            gui_disp_buf[i][j] = 0;
    }
    LCD_FillAll((unsigned char *)gui_disp_buf);
}

/*******************************************************************************
* Function Name  : LCD_Draw
* Description    : Draw LCD data from display buffer.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_Draw(void)
{
    LCD_FillAll((unsigned char *)gui_disp_buf);
}

/****************************************************************************
* Function Name  : LCD_UpdatePoint
* Description    : refresh the point in screen
* Input          : x      X-coordinate
y      Y-coordinate
* Output         : None
* Return         : None
****************************************************************************/
void LCD_UpdatePoint(unsigned int x, unsigned int y)
{
    unsigned char coll, colh, page;
    page = y / 8;
    coll = x & 0x0f;
    colh = x >> 4;

    LCD_WriteCmd(Set_Page_Addr_0 | page); // page no.
    LCD_WriteCmd(Set_ColH_Addr_0 | colh); // fixed col first addr
    LCD_WriteCmd(Set_ColL_Addr_0 | coll);
    LCD_WriteData(gui_disp_buf[page][x]);
}

/****************************************************************************
* Function Name  : LCD_PutChar
* Description    : output a char to screen
(the char only can be ' ','0'~'9','A'~'Z','a'~'z')
* Input          : x      X-coordinate
y      Y-coordinate
ch     character
* Output         : None
* Return         : 1    Success
0    Fail
****************************************************************************/
unsigned char LCD_PutChar(unsigned long x, unsigned long y, unsigned char ch)
{
    unsigned char data;
    unsigned char i;
    unsigned char j;

    if (x > (GUI_LCM_XMAX - 6))
        return (0);
    if (y > (GUI_LCM_YMAX - 8))
        return (0);

    if (ch >= 0x20)
        ch -= 0x20;
    else
        return (0);

    for (i = 0; i < FONT_VSIZE; i++)
    {
        data = FONTTYPE6_8[ch][i];

        for (j = 0; j < FONT_HSIZE; j++)
        {
            if ((data & BIT_MASK[j]) == 0)
                gui_disp_buf[y / 8][x] &= (~(0x01 << (y % 8)));
            else
                gui_disp_buf[y / 8][x] |= (0x01 << (y % 8));
            if (sUpdate_mode != 0)
            {
                LCD_UpdatePoint(x, y);
            }
            x++;
        }
        x -= FONT_HSIZE;
        y++;
    }

    return (1);
}

/****************************************************************************
* Function Name  : LCD_PutString
* Description    : output string to screen
* Input          : x      X-coordinate
y      Y-coordinate
str     pointer to string
* Output         : None
* Return         : None
****************************************************************************/
void LCD_PutString(unsigned long x, unsigned long y, char *str)
{
    while (1)
    {
        if ((*str) == '\0')
            break;
        if (LCD_PutChar(x, y, *str++) == 0)
            break;
        x += 6;
    }
}

/*******************************************************************************
* Function Name  : LCD_Write
* Description    : write data or command to lcd sdram
* Input          : command
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_WriteCmd(unsigned char command)
{
#ifdef SW_SPI
    unsigned char i;
#endif
    LCD_CD_LOW();
#ifdef SW_SPI
    for (i = 0; i < 8; i++)
    {
        if (command & (0x80 >> i))
            LCD_DATA_HIGH();
        else
            LCD_DATA_LOW();

        LCD_CLK_LOW();
        Delay();
        LCD_CLK_HIGH();
        Delay();
    }
#else
    LCD_SPI_SEND(command);
#endif
}

/*******************************************************************************
* Function Name  : LCD_Write
* Description    : write data or command to lcd sdram
* Input          : data: data or command
attribute: data or command
* Output         : None
* Return         : None
*******************************************************************************/
void LCD_WriteData(unsigned char data)
{
#ifdef SW_SPI
    unsigned char i;
#endif
    LCD_CD_HIGH();
#ifdef SW_SPI
    for (i = 0; i < 8; i++)
    {
        if (data & (0x80 >> i))
            LCD_DATA_HIGH();
        else
            LCD_DATA_LOW();

        LCD_CLK_LOW();
        Delay();
        LCD_CLK_HIGH();
        Delay();
    }
#else
    LCD_SPI_SEND(data);
#endif
}

/******************************************************************************/

const unsigned char Freescale_logo[] = {
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x7F,
    0x3F, 0x3F, 0x1F, 0x1F, 0x1F, 0x3F, 0x3F, 0x7F, 0x7F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF7, 0xF7, 0xE3, 0xE3, 0xC1, 0xC1, 0x80, 0x80, 0xC0, 0xC0, 0xE0, 0xE0, 0xF0, 0xF0,
    0xF8, 0x78, 0x7C, 0x3C, 0x3E, 0x1E, 0x1F, 0x0F, 0x07, 0x07, 0x03, 0x07, 0x07, 0x0F, 0x0F, 0x9F, 0x9F, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFC, 0xFC, 0xF8, 0xF8, 0xF8, 0xF0,
    0xF0, 0xF0, 0xF8, 0xF8, 0xFC, 0xBC, 0xBE, 0x1E, 0x1F, 0x0F, 0x0F, 0x07, 0x07, 0x03, 0x01, 0x01, 0x00, 0x80, 0x80,
    0xC1, 0xC1, 0xE3, 0xE3, 0xF3, 0xF7, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0x7F, 0x7F, 0x7F, 0x3F, 0x1F, 0x1F, 0x0F, 0x0F, 0x07, 0x07, 0x03, 0x03, 0x01, 0x03, 0x83, 0x83, 0xC7, 0xC7,
    0xEF, 0xEF, 0xFF, 0xFF, 0xFF, 0x7E, 0x7E, 0x7C, 0x7E, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0x7F, 0x7F, 0x3F, 0x3F, 0x1F, 0x1F, 0x0F, 0x0F, 0x07, 0x07, 0x0F, 0x0F, 0x0F, 0x0F, 0x9F, 0x9F, 0xBE, 0xFE, 0xFE,
    0xFC, 0xFC, 0xF8, 0xF8, 0xF8, 0xFC, 0xFC, 0xFE, 0xDE, 0xCF, 0xCF, 0x87, 0x87, 0x03, 0x03, 0x01, 0x01, 0x00, 0x80,
    0x80, 0xC0, 0xC0, 0xE0, 0xF0, 0xF0, 0xF9, 0xF9, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFD, 0xFC, 0xF8, 0xF8, 0xF8, 0xF0, 0xF0, 0xE0,
    0xF0, 0xF0, 0xF8, 0xF8, 0xBC, 0xBC, 0x1E, 0x1F, 0x0F, 0x0F, 0x07, 0x07, 0x03, 0x03, 0x01, 0x01, 0x00, 0x80, 0xC1,
    0xC1, 0xE3, 0xE3, 0xF7, 0xF7, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0x7F, 0x7F, 0x3F, 0x3F, 0x1F, 0x1F, 0x0F, 0x0F, 0x07, 0x07, 0x03, 0x03, 0x03, 0x03, 0x87, 0x87, 0xCF, 0xCF,
    0xFF, 0xFF, 0xFF, 0xFE, 0xFE, 0xFE, 0xFC, 0xFC, 0xFE, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xFE, 0xFC, 0xFC,
    0xF8, 0xF8, 0xF8, 0xF8, 0xFC, 0xFC, 0xFE, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
